/* Automatically created objects file */

func InitializeObjects()
{
	var Grass0001 = CreateObjectAbove(Grass, 132, 142);
	Grass0001->SetClrModulation(0xff0b1d05);
	var Grass0002 = CreateObjectAbove(Grass, 142, 139);
	Grass0002->SetClrModulation(0xff150b09);
	var Grass0003 = CreateObjectAbove(Grass, 125, 146);
	Grass0003->SetClrModulation(0xff06090a);
	var Grass0004 = CreateObjectAbove(Grass, 119, 151);
	Grass0004->SetClrModulation(0xff201308);
	var Grass0005 = CreateObjectAbove(Grass, 205, 146);
	Grass0005->SetClrModulation(0xff35020b);
	var Grass0006 = CreateObjectAbove(Grass, 179, 143);
	Grass0006->SetClrModulation(0xff3b0e05);
	var Grass0007 = CreateObjectAbove(Grass, 172, 141);
	Grass0007->SetClrModulation(0xff2e150e);
	CreateObjectAbove(Grass, 1360, 397);
	CreateObjectAbove(Grass, 1456, 325);
	CreateObjectAbove(Grass, 1462, 325);
	CreateObjectAbove(Grass, 1469, 325);
	CreateObjectAbove(Grass, 1475, 326);
	CreateObjectAbove(Grass, 1482, 326);
	CreateObjectAbove(Grass, 1269, 316);
	CreateObjectAbove(Grass, 1262, 315);
	CreateObjectAbove(Grass, 1281, 310);
	CreateObjectAbove(Grass, 1274, 312);
	CreateObjectAbove(Grass, 1254, 317);
	CreateObjectAbove(Grass, 1246, 315);
	CreateObjectAbove(Grass, 1243, 317);
	CreateObjectAbove(Grass, 32, 155);
	CreateObjectAbove(Grass, 38, 155);
	CreateObjectAbove(Grass, 27, 157);
	CreateObjectAbove(Grass, 1591, 349);
	CreateObjectAbove(Grass, 1598, 349);
	CreateObjectAbove(Grass, 1605, 349);
	CreateObjectAbove(Grass, 1612, 350);
	CreateObjectAbove(Grass, 1620, 353);
	CreateObjectAbove(Grass, 1725, 369);

	CreateObjectAbove(Tree_Coniferous, 359, 134);

	var Trunk0034 = CreateObjectAbove(Trunk, 149, 144);
	Trunk0034->SetCon(70);
	Trunk0034->SetR(10);
	Trunk0034->SetClrModulation(0xff808080);
	Trunk0034->SetPosition(149, 127);
	var Trunk0035 = CreateObjectAbove(Trunk, 119, 161);
	Trunk0035->SetCon(85);
	Trunk0035->SetClrModulation(0xff806640);

	var Tree_Coniferous0036 = CreateObjectAbove(Tree_Coniferous, 21, 159);
	Tree_Coniferous0036->SetCon(60);
	var Tree_Coniferous0040 = CreateObjectAbove(Tree_Coniferous, 50, 158);
	Tree_Coniferous0040->SetCon(80);
	var Tree_Coniferous0044 = CreateObjectAbove(Tree_Coniferous, 553, 70);
	Tree_Coniferous0044->SetCon(60);
	Tree_Coniferous0044->SetR(-20);
	Tree_Coniferous0044->SetPosition(553, 45);
	var Tree_Coniferous0048 = CreateObjectAbove(Tree_Coniferous, 663, 52);
	Tree_Coniferous0048->SetCon(80);
	CreateObjectAbove(Tree_Coniferous, 682, 52);
	CreateObjectAbove(Tree_Coniferous, 812, 57);

	CreateObjectAbove(Trunk, 778, 81);

	CreateObjectAbove(Tree_Coniferous, 547, 407);
	CreateObjectAbove(Tree_Coniferous, 494, 403);
	CreateObjectAbove(Tree_Coniferous, 416, 394);

	CreateObjectAbove(Trunk, 331, 364);
	var Trunk0074 = CreateObjectAbove(Trunk, 379, 260);
	Trunk0074->SetCon(70);
	Trunk0074->SetR(180);
	Trunk0074->SetPosition(379, 244);
	var Trunk0075 = CreateObjectAbove(Trunk, 353, 288);
	Trunk0075->SetCon(50);
	Trunk0075->SetR(140);
	Trunk0075->SetPosition(353, 280);

	var Branch0076 = CreateObjectAbove(Branch, 338, 290);
	Branch0076->SetR(140);
	Branch0076->SetPosition(338, 293);
	var Branch0077 = CreateObjectAbove(Branch, 618, 122);
	Branch0077->SetR(-140);
	Branch0077->SetPosition(618, 125);
	var Branch0078 = CreateObjectAbove(Branch, 585, 104);
	Branch0078->SetR(-150);
	Branch0078->SetPosition(585, 107);

	var Trunk0079 = CreateObjectAbove(Trunk, 643, 153);
	Trunk0079->SetCon(60);
	Trunk0079->SetR(-150);
	Trunk0079->SetPosition(643, 141);

	CreateObjectAbove(SproutBerryBush, 458, 406);

	var LargeCaveMushroom0084 = CreateObjectAbove(LargeCaveMushroom, 11, 306);
	LargeCaveMushroom0084->SetCon(60);
	LargeCaveMushroom0084->SetClrModulation(0xffe0ece4);

	CreateObjectAbove(Tree_Coniferous, 300, 134);

	var LargeCaveMushroom0096 = CreateObjectAbove(LargeCaveMushroom, 144, 513);
	LargeCaveMushroom0096->SetCon(60);
	LargeCaveMushroom0096->SetClrModulation(0xff788cc8);
	var LargeCaveMushroom0100 = CreateObjectAbove(LargeCaveMushroom, 1220, 489);
	LargeCaveMushroom0100->SetCon(50);
	LargeCaveMushroom0100->SetClrModulation(0xffe5e6ec);
	var LargeCaveMushroom0104 = CreateObjectAbove(LargeCaveMushroom, 176, 146);
	LargeCaveMushroom0104->SetCon(80);
	LargeCaveMushroom0104->SetClrModulation(0xff643c28);

	CreateObjectAbove(Tree_Coniferous, 1398, 324);
	CreateObjectAbove(Tree_Coniferous, 1328, 316);
	CreateObjectAbove(Tree_Coniferous, 1208, 311);
	CreateObjectAbove(Tree_Coniferous, 1168, 320);
	CreateObjectAbove(Tree_Coniferous, 1188, 317);
	CreateObjectAbove(Tree_Coniferous, 1128, 323);
	CreateObjectAbove(Tree_Coniferous, 1293, 314);
	CreateObjectAbove(Tree_Coniferous, 1337, 317);
	CreateObjectAbove(Tree_Coniferous, 1319, 312);
	CreateObjectAbove(Tree_Coniferous, 1368, 322);

	CreateObjectAbove(SproutBerryBush, 1258, 323);
	CreateObjectAbove(SproutBerryBush, 1579, 358);

	var Trunk0156 = CreateObjectAbove(Trunk, 1065, 369);
	Trunk0156->SetR(-30);
	Trunk0156->SetPosition(1065, 347);
	CreateObjectAbove(Trunk, 1522, 338);
	var Trunk0158 = CreateObjectAbove(Trunk, 1001, 520);
	Trunk0158->SetR(-80);
	Trunk0158->SetPosition(1001, 512);
	var Trunk0159 = CreateObjectAbove(Trunk, 680, 519);
	Trunk0159->SetR(65);
	Trunk0159->SetPosition(680, 507);

	CreateObjectAbove(BigRock, 523, 405);
	var BigRock0161 = CreateObjectAbove(BigRock, 711, 530);
	BigRock0161->SetR(60);
	BigRock0161->SetPosition(711, 527);
	CreateObjectAbove(BigRock, 1025, 560);
	CreateObjectAbove(BigRock, 338, 572);

	var Tree_Coniferous0164 = CreateObjectAbove(Tree_Coniferous, 261, 133);
	Tree_Coniferous0164->SetCon(70);
	var Tree_Coniferous0168 = CreateObjectAbove(Tree_Coniferous, 171, 197);
	Tree_Coniferous0168->SetCon(1);
	CreateObjectAbove(Tree_Coniferous, 1640, 367);
	CreateObjectAbove(Tree_Coniferous, 1700, 374);
	CreateObjectAbove(Tree_Coniferous, 1756, 370);
	CreateObjectAbove(Tree_Coniferous, 1673, 375);

	CreateObject(Rule_TeamAccount, 50, 50);

	CreateObject(Rule_BuyAtFlagpole, 50, 50);

	CreateObject(Rule_BuyAtFlagpole, 50, 50);

	var Chest0224 = CreateObjectAbove(Chest, 534, 543);
	var Chest0225 = CreateObjectAbove(Chest, 279, 585);
	var Chest0226 = CreateObjectAbove(Chest, 1185, 447);
	var Chest0227 = CreateObjectAbove(Chest, 1812, 549);

	var WindGenerator0190 = CreateObjectAbove(WindGenerator, 177, 144);
	WindGenerator0190->SetCon(80);
	WindGenerator0190->SetCategory(C4D_StaticBack);
	WindGenerator0190->SetR(92);
	WindGenerator0190->SetClrModulation(0xff503c28);
	WindGenerator0190->SetPosition(177, 134);
	WindGenerator0190->SetFlagRadius(0); // shouldn't have a power radius

	var Elevator0228 = CreateObjectAbove(Elevator, 69, 159);
	Elevator0228->CreateShaft(5);
	Elevator0228->SetCasePosition(152);

	var Clonk0236 = CreateObjectAbove(Clonk, 134, 396);
	Clonk0236->SetColor(0xfff0783c);
	Clonk0236->SetDir(DIR_Left);
	var Clonk0242 = CreateObjectAbove(Clonk, 149, 387);
	Clonk0242->SetColor(0xff78f03c);
	Clonk0242->SetDir(DIR_Left);
	var Clonk0248 = CreateObjectAbove(Clonk, 80, 387);
	Clonk0248->SetDir(DIR_Right);
	Clonk0248->SetColor(0xfff078f0);
	var Clonk0254 = CreateObjectAbove(Clonk, 520, 553);
	Clonk0254->Kill(Clonk0254, true);
	Clonk0254->SetDir(DIR_Right);
	Clonk0254->SetYDir(20);
	Clonk0254->SetColor(0xff503217);
	Clonk0254->DoEnergy(-50);

	Chest0225->CreateContents(Rock);
	Chest0225->CreateContents(Rock);
	Chest0226->CreateContents(Rock);
	Chest0226->CreateContents(Rock);

	Chest0224->CreateContents(Coal);
	Chest0224->CreateContents(Coal);

	Chest0224->CreateContents(Metal);
	Chest0224->CreateContents(Metal);
	Chest0226->CreateContents(Metal);

	Chest0225->CreateContents(Nugget);
	Chest0226->CreateContents(Nugget);
	Chest0226->CreateContents(Nugget);

	Chest0226->CreateContents(Wood);
	Chest0226->CreateContents(Wood);
	Chest0227->CreateContents(Wood);
	Chest0227->CreateContents(Wood);
	Chest0227->CreateContents(Wood);

	Chest0224->CreateContents(GoldBar);

	CreateObjectAbove(Mushroom, 790, 71);
	CreateObjectAbove(Mushroom, 799, 67);
	CreateObjectAbove(Mushroom, 672, 57);
	CreateObjectAbove(Mushroom, 252, 134);
	CreateObjectAbove(Mushroom, 137, 144);
	CreateObjectAbove(Mushroom, 590, 419);
	CreateObjectAbove(Mushroom, 623, 406);
	CreateObjectAbove(Mushroom, 548, 542);
	CreateObjectAbove(Mushroom, 84, 495);
	CreateObjectAbove(Mushroom, 159, 508);
	CreateObjectAbove(Mushroom, 223, 535);
	CreateObjectAbove(Mushroom, 1115, 479);
	CreateObjectAbove(Mushroom, 1079, 479);
	CreateObjectAbove(Mushroom, 1049, 478);
	CreateObjectAbove(Mushroom, 1115, 329);
	CreateObjectAbove(Mushroom, 1206, 488);
	Chest0225->CreateContents(Mushroom);

	Chest0225->CreateContents(Blunderbuss);

	Chest0225->CreateContents(LeadBullet);

	Chest0224->CreateContents(Dynamite);
	Chest0225->CreateContents(Dynamite);
	Chest0226->CreateContents(Dynamite);
	Chest0226->CreateContents(Dynamite);
	Chest0227->CreateContents(Dynamite);
	Chest0227->CreateContents(Dynamite);

	Chest0224->CreateContents(Firestone);
	Chest0224->CreateContents(Firestone);
	Chest0226->CreateContents(Firestone);

	return true;
}
